% The code computes Wasserstein distance on simulated graph 
% data and perform simulation study (Study 2) given in 
%
% Songdechakraiwut, T. Chung, M.K. 2020 Topological learning for 
% brain networks, arXiv: 2012.00675. 
%
% The paper is included with the Matlab zip file. 
% If you are using any part of the codes, please reference the above paper
%
%
% (C) 2020 Moo K. Chung, Tananun Songdechakraiwut
%          University of Wisconsin-Madison
%
% Contact mkchung@wisc.edu for the maintainance of codes and support.  
% Update history
%     2020 November 11, created - Tananun Songdechakraiwut
%     2020 December 10, 2021 June 28 edited Moo Chung


%MODULAR NETWORK SIMULATION based on the mixed-effects model
%
% parameters for random graphs used in the simulation
mu = 1;       %parameter in Gaussin noise N(mu, sigma^2)
sigma = .25;  %parameter in Gaussin noise N(mu, sigma^2)
nNodeG = 20;  %number of nodes in graph G
nNodeP = 30;  %number of nodes in graph P
nModuleG = 2; %number of clusters in graph G
nModuleP = 3; %number of clusters in graph P
p=0.9;        %probability of connection within module
              %1-p is the probability of connection between module

% simulate random graphs
G = random_modular_graph(nNodeG, nModuleG, p, mu, sigma); %netowrk with 2 modules
P = random_modular_graph(nNodeP, nModuleP, p, mu, sigma); %netowrk with 3 moduels

% display edge weight matrices
figure; subplot(1,2,1); imagesc(G); axis square; colorbar; 
subplot(1,2,2); imagesc(P); axis square; colorbar; 

%display graph showing clusters/modular structures
Gbinary = adj2bin(G,0.5);
Ggraph = graph(Gbinary)

Pbinary = adj2bin(P,0.5);
Pgraph = graph(Pbinary)

figure; subplot(1,2,1); plot(Ggraph, 'k','LineWidth', 2);
subplot(1,2,2); plot(Pgraph,'k','LineWidth', 2);

%Birth-death decomposition of Wasserstein distance
[Wb, Wd] = WS_decompose(G)
Wb % maximum spanning tree

%Computes Wasserstein distance betwen graph G and P
WS_distance(G,P) 

%Perform topological inference beteen the groups of graphs. 
nGroup1 = 4;
nGroup2 = 6;
g1 = cell(nGroup1, 1);
g2 = cell(nGroup2, 1);
figure
for k = 1:nGroup1
    g1{k} = random_modular_graph(nNodeG, nModuleG, p, mu, sigma);
    subplot(2,2,k); imagesc(g1{k});
end

figure;
for k = 1:nGroup2
    g2{k} = random_modular_graph(nNodeP, nModuleP, p, mu, sigma);
    subplot(2,3,k); imagesc(g2{k});
end

distancemat = WS_distancemat(g1, g2);
figure; imagesc(distancemat); colorbar


%Ratio statitic = within-group Wasserstein distance / between-group Wasserstein distance
observedStat = WS_ratio(distancemat, nGroup1, nGroup2, @between_over_within)

%Transposition test
nTrans = 500000; % number of transpositions
permNo = 500; % intermix random permutations for every 'permNo' number of transpositions

[ReampledStat, ~] = WS_transpositions(distancemat, nGroup1, nGroup2, nTrans, ...
        permNo, @between_over_within); 
  
%Online p-value computation
pvalues = online_pvalues(ReampledStat, observedStat);
figure; plot(pvalues,'k','LineWidth',2)
pvalues(end)


     
    